/**
 * This is the main libmarshal library test program.
 *
 * When miniprintf.c is used, set Serial Port Baudrate to 57600
 *
 * This program and library are MIT licensed.
 * See terms below.
 */
#include <stdio.h>
#include "propeller.h"
#include "marshal.h"

void convert_test(int type, void *val, char *str);

int main(void)
{
    int n = 0, len = 0;

    int8_t      int8_val = 0x55;
    uint8_t     uint8_val = 0xAA;
    int16_t     int16_val = 0x1234;
    uint16_t    uint16_val = 0x5678;
    int32_t     int32_val  = 0x1deadbee;
    uint32_t    uint32_val = 0xcafebabe;

    char        string[10];

    putchar('\0');                  // clear any Quickstart garbage
    waitcnt(CLKFREQ/2+CNT);         // pause for the cause
    setvbuf(stdout, 0, _IONBF, 0);  // turn off IO buffering

    convert_test(INT8,   &int8_val,   string);
    convert_test(UINT8,  &uint8_val,  string);
    convert_test(INT16,  &int16_val,  string);
    convert_test(UINT16, &uint16_val, string);
    convert_test(INT32,  &int32_val,  string);
    convert_test(UINT32, &uint32_val, string);

    return 0;
}


void convert_test(int type, void *val, char *str)
{
    int n = 0, len = 0;
    switch(type) {
        case INT8:
            len = STRLEN8;
            printf("INT8   type. ");
            int8tostr(*(int8_t*) val, str);
            printf("Value %8x %8x, String: ", *(int8_t*) val, strtoint8(str));
        break;
        case UINT8:
            len = STRLEN8;
            printf("UINT8  type. ");
            uint8tostr(*(uint8_t*) val, str);
            printf("Value %8x %8x, String: ", *(uint8_t*) val, strtouint8(str));
        break;
        case INT16:
            len = STRLEN16;
            printf("INT16  type. ");
            int16tostr(*(int16_t*) val, str);
            printf("Value %8x %8x, String: ", *(int16_t*) val, strtoint16(str));
        break;
        case UINT16:
            len = STRLEN16;
            printf("UINT16 type. ");
            uint16tostr(*(uint16_t*) val, str);
            printf("Value %8x %8x, String: ", *(uint16_t*) val, strtouint16(str));
        break;
        case INT32:
            len = STRLEN32;
            printf("INT32  type. ");
            int32tostr(*(int32_t*) val, str);
            printf("Value %8x %8x, String: ", *(int32_t*) val, strtoint32(str));
        break;
        case UINT32:
            len = STRLEN32;
            printf("UINT32 type. ");
            uint32tostr(*(uint32_t*) val, str);
            printf("Value %8x %8x, String: ", *(uint32_t*) val, strtouint32(str));
        break;
        default:
        break;
    }
    for(n = 0; n < len; n++) printf("%x ",str[n]);
    printf("\n");
}

/*
Copyright (c) 2013, Steve Denson
+--------------------------------------------------------------------
|  TERMS OF USE: MIT License
+--------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+------------------------------------------------------------------
*/