/**
 * @file marshal.h
 */
#ifndef __marshall_h
#define __marshall_h

#include <stdint.h>

enum menum_types
{
    INT8 = 0xa1,
    UINT8,
    INT16,
    UINT16,
    INT32,
    UINT32
};

enum string_size_enum
{
    STRLEN8  = 2,
    STRLEN16 = 3,
    STRLEN32 = 5,
    STRLENLAST
};

/**
 * Encode a signed 8 bit word to a string having an INT8 preamble byte
 * @param [in] value The value to marshal.
 * @param [in,out] string The encoded 2 byte string. String storage must be provided by the caller.
 */
void int8tostr(int8_t value, char *string);

/**
 * Encode an unsigned 8 bit word to a string having a UINT8 preamble byte
 * @param [in] value The value to marshal.
 * @param [in,out] string The encoded 2 byte string. String storage must be provided by the caller.
 */
void uint8tostr(uint8_t value, char *string);

/**
 * Encode a signed 16 bit word to a string having an INT16 preamble byte
 * @param [in] value The value to marshal.
 * @param [in,out] string The encoded 3 byte string. String storage must be provided by the caller.
 */
void int16tostr(int16_t value, char *string);

/**
 * Encode an unsigned 16 bit word to a string having a UINT16 preamble byte
 * @param [in] value The value to marshal.
 * @param [in,out] string The encoded 3 byte string. String storage must be provided by the caller.
 */
void uint16tostr(uint16_t value, char *string);

/**
 * Encode a signed 32 bit word to a string having an INT32 preamble byte
 * @param [in] value The value to marshal.
 * @param [in,out] string The encoded 5 byte string. String storage must be provided by the caller.
 */
void int32tostr(int32_t value, char *string);

/**
 * Encode an unsigned 32 bit word to a string having a UINT32 preamble byte
 * @param [in] value The value to encode.
 * @param [in,out] string The encoded 5 byte string. String storage must be provided by the caller.
 */
void uint32tostr(uint32_t value, char *string);


/**
 * Decode a signed 8 bit word from an encoded string
 * @param [in] string The 2 byte string to decode.
 * @return value The decoded value.
 */
int8_t strtoint8(char string[STRLEN8]);

/**
 * Decode an unsigned 8 bit word from an encoded string
 * @param [in] string The 2 byte string to decode.
 * @return value The decoded value.
 */
uint8_t strtouint8(char string[STRLEN8]);

/**
 * Decode a signed 16 bit word from an encoded string
 * @param [in] string The 3 byte string to decode.
 * @return value The decoded value.
 */
int16_t strtoint16(char string[STRLEN16]);

/**
 * Decode an unsigned 16 bit word from an encoded string
 * @param [in] string The 3 byte string to decode.
 * @return value The decoded value.
 */
uint16_t strtouint16(char string[STRLEN16]);

/**
 * Decode a signed 32 bit word from an encoded string
 * @param [in] string The 5 byte string to decode.
 * @return value The decoded value.
 */
int32_t strtoint32(char string[STRLEN32]);

/**
 * Decode an unsigned 32 bit word from an encoded string
 * @param [in] string The 5 byte string to decode.
 * @return value The decoded value.
 */
uint32_t strtouint32(char string[STRLEN32]);

/*
Copyright (c) 2013, Steve Denson
+--------------------------------------------------------------------
|  TERMS OF USE: MIT License
+--------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+------------------------------------------------------------------
*/
#endif